from setuptools import setup

setup(name='cwcutils',
      version='0.1.2',
      description='Useful functions over time',
      url='',
      author='Chaowei Chen',
      author_email='willhyper@gmail.com',
      license='MIT',
      packages=['cwcutils','cwcutils.dec']
      )